//==============================================================================
//
// MACTek - Wireless HART Adapter DD
//
//  MACTek Corporation All rights reserved. Proprietary and Confidential
// 
// No part of this software may be disclosed or
// distributed in any form or by any means without the prior written
// consent of MACTek Corporation
//
//==============================================================================

/*****************************************************************************
 * Build Specific Settings
 *****************************************************************************
 */

// The version number for the DD
#define DD_VERSION                      2.25

MANUFACTURER 0xe0, DEVICE_TYPE 0xac, DEVICE_REVISION 2, DD_REVISION 2
#define DEVICE_NAME "Bullet"

/*****************************************************************************
 * Include Files
 *****************************************************************************
 */


#include "macros2.h"
#include "methods.h"
#include "DD_alerts.dd"
#include "DD_burst.dd"
#include "DD_power.dd"
#include "DD_subdevices.dd"
#include "DD_variables.dd"
#include "DD_wireless.dd"


/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */

#define UPDATE_IN_PROGRESS      8
#define DEVICE_BUSY             32
#define MAXIMUM_RESET_RETRIES   10


/*****************************************************************************
 * Definitions (#define, typedef, enum)
 *****************************************************************************
 */


/*****************************************************************************
 * Common Tables
 */

IMPORT STANDARD _TABLES, DEVICE_REVISION 20, DD_REVISION 4
{
    // Import all so that we get all device types for subdevices attached to the adapter.
    EVERYTHING;

    REDEFINITIONS
    {
        /* 
         * This must be redefined if the Device Type is not listed
         * in Common Tables DD
         */
        VARIABLE device_type
        {
            REDEFINE TYPE ENUMERATED (2)
            {
                // replace the 0xf9f3 with your expanded device type code and appropriete label
                { 0xE0AC,  "Wireless Adapter" }
            }
        }

        /*
         * This must be redefined if the Manufacturer code is not listed
         * in the Common Tables DD
         */
        VARIABLE manufacturer_id
        {
            REDEFINE TYPE ENUMERATED (2)
            {
                // replace the 0x00f9 with your manufactures id and appropriate label
                { 0x6019,  [MACTek] } 
            }
        }

        /*
         * This must be redefined if the Manufacturer code is not listed
         * in the Common Tables DD
         */
        VARIABLE private_label_distributor
        {
            REDEFINE TYPE ENUMERATED (2)
            {                
                { 0x6019,  [MACTek] } 
            }
        }

        VARIABLE burst_mode_select
        {
            REDEFINE TYPE ENUMERATED
            {
                BURST_MODE_CONTROL_CODE(0),  /* Off */                
                { 2,  "Enabled",  "Enabled on WirelessHART device connection only" }             
            }
        }

        VARIABLE event_notification_control_n
        {
           REDEFINE TYPE ENUMERATED
            {
                BURST_MODE_CONTROL_CODE(0),  /* Off */                
                { 2,  "Enabled",  "Enabled on WirelessHART device connection only" }             
        

            }
        }        

       /* 
	    * Remove the following if the device supports trends.  Corresponding
        * commands will also need to be imported from the COMMON_PRACTICE DD.
	    */
        DELETE VARIABLE trend_digital_units;
        DELETE VARIABLE trend_classification;
        DELETE VARIABLE trend_control_codes;
        DELETE VARIABLE trend_control;

        //Commands 821,822 are optional for wireless devices - remove if device does not support
        DELETE VARIABLE network_access_mode;

         //Command 89 is not typically supported by wireless adapters
        DELETE VARIABLE time_set;     
    }
}

/*****************************************************************************
 * Universal Commmands
 */

IMPORT STANDARD _UNIVERSAL, DEVICE_REVISION 7, DD_REVISION 2
{
    // All HART devices must support all Universal Commands
    EVERYTHING;

    REDEFINITIONS
    {   
        REDEFINE COMMAND read_unique_identifier 
        {
            NUMBER 0;
            OPERATION READ;
            TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    IDENTIFY_REPLY_7_2
                }
            }
            RESPONSE_CODES
            {
                0,  SUCCESS,    [no_command_specific_errors];
            }
        }
             
        COMMAND read_dynamic_variables_and_pv_current
        {
            REDEFINE TRANSACTION
            {
                REQUEST {}
                REPLY
                {
                    response_code, device_status,
                    PV.DAQ.ANALOG_VALUE,
                    PV.DEVICE_VARIABLE.DIGITAL_UNITS, PV.DEVICE_VARIABLE.DIGITAL_VALUE                    
                }
            }
        }
        
 
        /* Redefine the command 48 status to match
           the device implementation.  The same should
           also be done for device_specific_status_0_mask
           and device_specific_status_0_latched_value
           example for 1 byte follows: */
        VARIABLE device_specific_status_0
        {
            REDEFINE LABEL "Device Specific Status 0";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Stack Overflow", MISC },
                { 0x02, "Burst Messages Dropped", MISC },
                { 0x04, "Incorrect Power Mode", MISC },
                { 0x08, "Bad Loop Switch Parameters", MISC },
                { 0x10, "Bad Burst Configuration", MISC },
                { 0x20, "Subdevice Dropped", MISC },
                { 0x40, "Duplicate Poll Addr Detected", MISC },
                { 0x80, "Wired Burst Mode Detected", MISC }
            }
        }
        VARIABLE device_specific_status_1
        {
            REDEFINE LABEL "Device Specific Status 1";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Wireless Join Failed", MISC }
                { 0x02, "Loop Switch Cutoff Voltage Tripped", MISC }
            }
        }

        //Remove the command 48 status bytes that are not supported by the adapter
        //See also command 48 related items in the common practice imports.
        COMMAND read_additional_device_status
        {
            REDEFINE TRANSACTION 0
            {
                REQUEST
                {}
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0, device_specific_status_1, device_specific_status_2,
                    device_specific_status_3, device_specific_status_4, device_specific_status_5,
        
                    extended_fld_device_status, 0x00, 
                    
                    standardized_status_0, standardized_status_1, analog_channel_saturated1,
                    standardized_status_2, standardized_status_3
                }
            }

            REDEFINE TRANSACTION 1
            {
                REQUEST
                {
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),
        
                    extended_fld_device_status (INFO), 0x00, 
                    
                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO)
                }
                REPLY
                {
                    response_code, device_status, 
                    device_specific_status_0 (INFO), device_specific_status_1 (INFO), device_specific_status_2 (INFO),
                    device_specific_status_3 (INFO), device_specific_status_4 (INFO), device_specific_status_5 (INFO),
        
                    extended_fld_device_status (INFO), 0x00, 
                    
                    standardized_status_0 (INFO), standardized_status_1 (INFO), analog_channel_saturated1 (INFO),
                    standardized_status_2 (INFO), standardized_status_3 (INFO)
                }
            }
        }
    }
}


/*****************************************************************************
 * PV, when the primary variable is not mappable we can import the PV
 * standard DD to take care of the dynamic variable array, loop current
 * and ranging functions
 */

IMPORT STANDARD _PV, DEVICE_REVISION 1, DD_REVISION 1
{
    EVERYTHING;
    REDEFINITIONS
    {



        /* Redefine PV so that only non-special variables can be mapped.
           This is necessary for hosts that do not support re-entrant
           array references. */
        COLLECTION OF VARIABLE primary_variable
        {
            MEMBERS
            {
                REDEFINE DEVICE_VARIABLE, mappableVariables[0];
            }
        }

        VARIABLE transfer_function
        {            
            REDEFINE HANDLING READ & WRITE;
            REDEFINE TYPE ENUMERATED
            {
                { 0,   [linear],     [linear_help] },
                { 4,   "Special Curve",     [special_curve_help] }
            }
        }
        

        COLLECTION OF VARIABLE scaling
        {
            REDEFINE MEMBERS
            {
                /* Range Information */
                RANGE_UNITS,            deviceVariables[0].DIGITAL_UNITS, [range_units], [range_units_help];
                PERCENT_RANGE,          percentRange,       [percent_of_range],     [percent_of_range_help];
                TRANSFER_FUNCTION,      transfer_function,  "Linearization Mode",   [transfer_function_help];
                UPPER_RANGE_VALUE,      upperRange_value,   "Upper Range Value (20 mA)",  [upper_range_value_help];
                LOWER_RANGE_VALUE,      lowerRange_value,   "Lower Range Value (4 mA)",  [lower_range_value_help];
            }
        }
    }
}


ARRAY OF COLLECTION mappableVariables
{
    ELEMENTS
    {
          0, devVar0,     "Device Variable 0";
    }
}


/*****************************************************************************
 * Common Practice Commands
 */

IMPORT STANDARD _COMMON_PRACTICE, DEVICE_REVISION 9, DD_REVISION 2
{
    //For a wireless adapter, these are the minimal commands required (in the device) for compliance
    //commented items are required in the device, but not modeled in a DD
    COMMAND write_pv_damping_value;                     // Command 34
    COMMAND write_pv_range_values;                      // Command 35
    COMMAND enter_exit_fixed_pv_current_mode;           // Command 40
    COMMAND perform_self_test;                          // Command 41 
    COMMAND perform_device_reset;                       // Command 42 
    COMMAND write_pv_units;                             // Command 44
    COMMAND trim_pv_current_dac_zero;                   // Command 45
    COMMAND trim_pv_current_dac_gain;                   // Command 46
    COMMAND write_pv_transfer_function;                 // Command 47
    COMMAND write_device_variable_units;                // Command 53
    COMMAND read_device_variable_information;           // Command 54                                                            
    COMMAND write_number_of_response_preambles;         // Command 59
    COMMAND read_io_system_capabilities;                // Command 74

    //COMMAND read_io_channel_stats;                      // Command 85

    COMMAND write_io_system_master_mode;                // Command 87
    COMMAND write_io_system_retry_count;                // Command 88
    COMMAND read_device_communication_statistics;       // Command 95
    COMMAND flush_delayed_responses;                    // Command 106

    COMMAND read_country_code;                          // Command 512
    COMMAND write_country_code;                         // Command 513

    VARIABLE max_io_cards;                              // for Command 74
    VARIABLE max_channels_per_io_card;                  // for Command 74
    VARIABLE max_sub_dev_per_channel;                   // for Command 74
    VARIABLE devices_detected;                          // for Command 74
    VARIABLE max_dr;                                    // for Command 74
    VARIABLE retry_count;                               // for Command 74, 88
    VARIABLE device_variable_code;                      // for Command 79    
    VARIABLE device_variable_trim_code;                 // for Commands 80-83
                                                        // 
    #if __TOKVER__ >= 800
    VARIABLE current_date;                              // for Command 90
    VARIABLE current_time;                              // for Command 90
    VARIABLE last_clock_date;                           // for Command 90
    VARIABLE last_clock_time;                           // for Command 90
    #endif

    VARIABLE STX_count;                                 // for Command 95
    VARIABLE ACK_count;                                 // for Command 95
    VARIABLE BACK_count;                                // for Command 95
    VARIABLE total_number_burst_messages;               // for Command 105
    VARIABLE subdev_IO_card;                            // for collection subdev_identity
    VARIABLE subdev_channel;                            // for collection subdev_identity
    VARIABLE subdev_device_id;                          // for collection subdev_identity
    VARIABLE subdev_universal_command_revision;         // for collection subdev_identity
    VARIABLE subdev_long_tag;                           // for collection subdev_identity
    VARIABLE stx_sent_to_subdev;                        // for collection subdev_stats
    VARIABLE ack_received_from_subdev;                  // for collection subdev_stats
    VARIABLE back_received_from_subdev;                 // for collection subdev_stats
    VARIABLE io_channel_stx_sent;                       // for collection io_stats
    VARIABLE io_channel_ack_received;                   // for collection io_stats
    VARIABLE io_channel_ostx_received;                  // for collection io_stats
    VARIABLE io_channel_oack_received;                  // for collection io_stats
    VARIABLE io_channel_back_received;                  // for collection io_stats
    VARIABLE burst_command_number;                      // for collection burst_message_collection
    VARIABLE trigger_level;                             // for collection burst_message_collection

    VARIABLE number_events_supported;                   // for Command 115
    VARIABLE device_specific_status_0_mask;             // for collection event_mask
    VARIABLE device_specific_status_1_mask;             // for collection event_mask
    VARIABLE device_specific_status_2_mask;             // for collection event_mask
    VARIABLE device_specific_status_3_mask;             // for collection event_mask
    VARIABLE device_specific_status_4_mask;             // for collection event_mask
    VARIABLE device_specific_status_5_mask;             // for collection event_mask

    /*
     * Adapters handle events on behalf of subdevices,
     * so ALL status masks must be imported so that
     * subdevices with any number of status bytes can
     * be supported.
     */
    VARIABLE device_specific_status_14_mask;            // for collection event_mask
    VARIABLE device_specific_status_15_mask;            // for collection event_mask
    VARIABLE device_specific_status_16_mask;            // for collection event_mask
    VARIABLE device_specific_status_17_mask;            // for collection event_mask
    VARIABLE device_specific_status_18_mask;            // for collection event_mask
    VARIABLE device_specific_status_19_mask;            // for collection event_mask
    VARIABLE device_specific_status_20_mask;            // for collection event_mask
    VARIABLE device_specific_status_21_mask;            // for collection event_mask
    VARIABLE device_specific_status_22_mask;            // for collection event_mask
    VARIABLE device_specific_status_23_mask;            // for collection event_mask
    VARIABLE device_specific_status_24_mask;            // for collection event_mask
    VARIABLE device_specific_status_0_latched_value;    // for collection event_report
    VARIABLE device_specific_status_1_latched_value;    // for collection event_report
    VARIABLE device_specific_status_2_latched_value;    // for collection event_report
    VARIABLE device_specific_status_3_latched_value;    // for collection event_report
    VARIABLE device_specific_status_4_latched_value;    // for collection event_report
    VARIABLE device_specific_status_5_latched_value;    // for collection event_report

    /*
     * Adapters handle events on behalf of subdevices,
     * so ALL status masks must be imported so that
     * subdevices with any number of status bytes can
     * be supported.
     */
    VARIABLE device_specific_status_14_latched_value;   // for collection event_report
    VARIABLE device_specific_status_15_latched_value;   // for collection event_report
    VARIABLE device_specific_status_16_latched_value;   // for collection event_report
    VARIABLE device_specific_status_17_latched_value;   // for collection event_report
    VARIABLE device_specific_status_18_latched_value;   // for collection event_report
    VARIABLE device_specific_status_19_latched_value;   // for collection event_report
    VARIABLE device_specific_status_20_latched_value;   // for collection event_report
    VARIABLE device_specific_status_21_latched_value;   // for collection event_report
    VARIABLE device_specific_status_22_latched_value;   // for collection event_report
    VARIABLE device_specific_status_23_latched_value;   // for collection event_report
    VARIABLE device_specific_status_24_latched_value;   // for collection event_report
    VARIABLE config_change_counter_latched_value;       // for collection event_report

    VARIABLE constOne;                                  // for command 101,102
    VARIABLE constZero;                                 // for command 101,102
    VARIABLE subdevice_missing;                         // for command 101,102

    //UNIT burst_trigger_units_relation;
    
    COLLECTION subdev_identity;                         // for collection sub_device
    COLLECTION subdev_stats;                            // for collection sub_device
    COLLECTION sub_device;                              // for LIST subdevice_list
    COLLECTION io_stats;                                // for LIST channel_list

    METHOD leave_fixed_current_mode;
    METHOD return_to_normal;
    METHOD warning_message;
    METHOD device_self_test;
    METHOD device_variable_trim;
    METHOD zero_trim;
    METHOD device_variable_zero_trim;
    METHOD trim_warning;
    METHOD device_variable_trim_reset;
    METHOD transmitter_dac_trim;
    
    REDEFINITIONS
    {
        // Define how many burst messages this adapter supports
        COLLECTION OF VARIABLE subdev_identity
        {
            //A minimum of 4 are required
            REDEFINE MEMBERS
            {
                IO_CARD,            subdev_IO_card,                        "IO Card";
                CHANNEL,            subdev_channel,                        "Channel";
                MANUFID,            subdev_manufacturer_id,                "Mfg ID";
                EXP_DEV_TYPE,       subdev_expanded_device_type,           "Device Type";
                DEV_ID,             subdev_device_id,                      "Device ID";                
                UNIV_CMD_REV,       subdev_universal_command_revision,     "Univ Rev";
                LONG_TAG,           subdev_long_tag,                       "Long Tag";
                POLL_ADDR,          subdev_poll_addr,                      "Polling Address";
                LOOP_CUR,           subdev_loop_cur,                       "Loop Current Mode";
                SHORT_TAG,          subdev_short_tag,                      "Tag";
                DEVICE_STATUS,      subdev_device_stat,                    "Device Status";
                CMD_SUPPORT,        subdev_command_support,                "Command Support";
                COMM_STATUS,        subdev_comm_stat,                      "Comm Status";
            }
        }

        // Define the list of commands that can be burst by the adapter (add any device specific commands too)
        VARIABLE burst_command_number
        {
            REDEFINE TYPE ENUMERATED (2)
            {
                {1,  [read_primary_variable], [pv_help] }, 
                {2,  [read_percent_range_and_current], [percent_rnge_help] }, 
                {3,  [read_dyn_vars_and_current], [all_process_var_help] }, 
                {9,  [read_device_variables_with_status], [read_device_variables_with_status_help] }, 
                {33, [read_device_variables],[read_device_variables_help] },
                {48, "Cmd 48: Read Additional Device Status", "Read Additional Device - This command reads additional device status information"}                
                {778, "Cmd 778: Read Battery Life", "Read Battery Life - This command reads the battery life of the device"}                 
            }
        }

        /* The event masks and reported values should be
           redefined similar to the command 48 byte
           redefinitions in the UNIVERSAL imports. */
        VARIABLE device_specific_status_0_mask
        {
            REDEFINE LABEL "Status example 1 Mask";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Example condition 1", MISC },
                { 0x02, "Example condition 2", MISC },
                { 0x04, "Example condition 3", MISC },
                { 0x08, "Example condition 4", MISC },
                { 0x10, "Example condition 5", MISC },
                { 0x20, "Example condition 6", MISC },
                { 0x40, "Example condition 7", MISC },
                { 0x80, "Example condition 8", MISC }
            }
        }

        VARIABLE device_specific_status_0_latched_value
        {
            REDEFINE LABEL "Latched Status example 1";
            REDEFINE TYPE BIT_ENUMERATED
            {
                { 0x01, "Example condition 1", MISC },
                { 0x02, "Example condition 2", MISC },
                { 0x04, "Example condition 3", MISC },
                { 0x08, "Example condition 4", MISC },
                { 0x10, "Example condition 5", MISC },
                { 0x20, "Example condition 6", MISC },
                { 0x40, "Example condition 7", MISC },
                { 0x80, "Example condition 8", MISC }
            }
        }
    }
}


IMPORT MANUFACTURER 0x00, DEVICE_TYPE 0x18, DEVICE_REVISION 1, DD_REVISION 2

{
    EVERYTHING;

    REDEFINITIONS
    {
        // Commands 821,822 are optional for adapters - remove if device does not support
        DELETE COMMAND write_network_access_mode;
        DELETE COMMAND read_network_access_mode;
        DELETE COMMAND read_network_tag;
        DELETE COMMAND write_radio_power_output;
        DELETE COMMAND read_radio_output_power;
        DELETE COMMAND read_wireless_module_revision; 
        DELETE VARIABLE radio_power_output;
    }
}


VARIABLE device_description_version2
{
    LABEL "DD Version";
    HELP "The version number for the device description";
    CLASS LOCAL;
    TYPE FLOAT
    {
        DISPLAY_FORMAT "1.2f" ;
    }
    DEFAULT_VALUE DD_VERSION;
}


VARIABLE current_time1 
{
    LABEL "Current Time";
    HELP "Current Time based upon the real-time clock in the field device";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE ASCII(8);
}


VARIABLE last_clock_time1
{
    LABEL "Set Clock Time";
    HELP "Time to set the real time clock in the field device to, or time that the real-time clock was last set";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE ASCII(8);
}


VARIABLE software_build_date
{
    LABEL "Software Build Date";
    HELP "Software Build Date";
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (20);
}


VARIABLE dust_software_major
{
    LABEL "Radio Software Rev Major";
    HELP "Radio Software Revision Major";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE dust_software_minor
{
    LABEL "Radio Software Rev Minor";
    HELP "Radio Software Revision Minor";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE dust_software_patch
{
    LABEL "Radio Software Rev Patch";
    HELP "Radio Software Revision Patch";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE dust_software_build
{
    LABEL "Radio Software Rev Build";
    HELP "Radio Software Revision Build";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}


VARIABLE software_revision_internal
{
    LABEL "Software Build";
    HELP "Software Build";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}


VARIABLE software_bootloader_revision
{
    LABEL "Software Bootloader Revision";
    HELP "Internal Software Bootloader Revision";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE device_serial_num
{
    LABEL "Device Serial Number";
    HELP "Device Serial Number";
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (16);
}


VARIABLE device_model_num
{
    LABEL "Model Number";
    HELP "Model Number";
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (16);
}


VARIABLE device_build_date
{
    LABEL "Production Date";
    HELP "Production Date";
    CLASS DEVICE;
    HANDLING READ;
    TYPE DATE;
}


VARIABLE device_lot_num
{
    LABEL "Lot Num";
    HELP "Lot Num";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER (4);
}


VARIABLE device_mac_address
{
    LABEL "MAC Address";
    HELP "MAC Address";
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (23);
}


VARIABLE system_uptime
{
    LABEL "System Uptime";
    HELP "Time in seconds since the system was powered on";
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    CONSTANT_UNIT [sec];
    TYPE UNSIGNED_INTEGER (4);
}


VARIABLE write_protect_mode
{
    LABEL "Write Protect";
    HELP "Write Protect";
    HANDLING READ & WRITE;
    CLASS DEVICE;
    TYPE ENUMERATED
    {
        { 0,   "Disabled" },
        { 1,   "Enabled" }
    }
}


VARIABLE user_password
{
    LABEL "User Password";
    HELP "User Password";
    HANDLING WRITE;
    TYPE PASSWORD (10) ;
}


VARIABLE license_key_ascii
{
    LABEL "License Key";
    HELP "License Key";
    CLASS DEVICE;
    HANDLING READ;
    TYPE ASCII (35);
}

VARIABLE license_key1
{
    LABEL "Key 1";
    HELP  "First key word for " DEVICE_NAME " License";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
    DEFAULT_VALUE 0;
}

VARIABLE license_key2
{
    LABEL "Key 2";
    HELP  "Second key word for " DEVICE_NAME " License";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
    DEFAULT_VALUE 0;
}

VARIABLE license_key3
{
    LABEL "Key 3";
    HELP  "Third key word for " DEVICE_NAME " License";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
    DEFAULT_VALUE 0;
}

VARIABLE license_key4
{
    LABEL "Key 4";
    HELP  "Fourth key word for " DEVICE_NAME " License";
    CLASS DEVICE;
    HANDLING READ & WRITE;
    TYPE UNSIGNED_INTEGER (4)
    {
       DISPLAY_FORMAT "08X";
       EDIT_FORMAT "08X";
    }
    DEFAULT_VALUE 0;
}


VARIABLE license_key_subdevices
{
    LABEL "Subdevices Supported";
    HELP  "The number of subdevices supported with this license key";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE license_key_burst
{
    LABEL "Burst Messages Supported";
    HELP  "The number of burst messages supported with this license key";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE license_key_event
{
    LABEL "Event Messages Supported";
    HELP  "The number of event messages supported with this license key";
    CLASS DEVICE;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER;
}


VARIABLE license_key_power
{
    LABEL "Power Modes Supported";
    HELP  "The power modes supported with this license key";
    CLASS DEVICE;
    HANDLING READ;    
    TYPE BIT_ENUMERATED(2)
    {
        { 0x0008, "StepVolt Supported", DEVICE },
        { 0x0010, "Direct Power Supported", DEVICE },
        { 0x0100, "Integral Resistor Supported", DEVICE },
        { 0x0200, "Field Device Power Control Supported", DEVICE }
    }
}


METHOD set_license_key
{
    LABEL "Set License Key";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];        
        int wReply;
        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        do
        {    
            GET_DEV_VAR_VALUE("Enter the License Key:\n(part 1 of 4)", license_key1);
            GET_DEV_VAR_VALUE("Enter the License Key:\n(part 2 of 4)", license_key2);
            GET_DEV_VAR_VALUE("Enter the License Key:\n(part 3 of 4)", license_key3);
            GET_DEV_VAR_VALUE("Enter the License Key:\n(part 4 of 4)", license_key4);                

            wReply = SELECT_FROM_LIST("New License Key: %[08X]{license_key1}-%[08X]{license_key2}-%[08X]{license_key3}-%[08X]{license_key4}\n\n.","Accept new license key;Re-enter license key");

        } while( wReply != 0 );

        send(164,status);
        if (status[STATUS_RESPONSE_CODE]==0)
        {            
            // Update the count variables used by the burst, event and subdevice menus
            send(163,status);
            send(74,status);  
            send(138,status);  
            send_trans(105,0,status);  
            send_trans(193,0,status);   
            ACKNOWLEDGE("License Key Accepted");            
        }
        else
        {
            send(163,status);
            ACKNOWLEDGE("License Key Invalid");  
        }   
    }
}

COMMAND read_license_key
{
    NUMBER 163;
    OPERATION READ;
    TRANSACTION
    {     
        REQUEST
        {           
        }
        REPLY
        {
            response_code, device_status,
            license_key1, license_key2, license_key3, license_key4,
            license_key_subdevices, license_key_burst, license_key_event, license_key_power,
            license_key_ascii
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
    }
}


COMMAND write_license_key
{
    NUMBER 164;
    OPERATION WRITE;
    TRANSACTION
    {     
        REQUEST
        {  
            license_key1, license_key2, license_key3, license_key4      
        }
        REPLY
        {
            response_code, device_status,
            license_key1, license_key2, license_key3, license_key4,
            license_key_subdevices, license_key_burst, license_key_event, license_key_power            
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
    }
}


COMMAND read_real_time_clock_ascii
{
    NUMBER 190;
    OPERATION READ;
    TRANSACTION
    {     
        REQUEST
        {           
        }
        REPLY
        {
            response_code, device_status,
            current_date, current_time1, last_clock_date, 
            last_clock_time1, real_time_clock_flag
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        32, MODE_ERROR,       [busy];
        33, MODE_ERROR,       [delayed_response_initiated];
        34, MODE_ERROR,       [delayed_response_running];
        35, MODE_ERROR,       [delayed_response_dead];
        36, MODE_ERROR,       [delayed_response_conflict];
    }
}


COMMAND InternalSoftwareRevRead1
{
    NUMBER 168;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            dummy_variable, software_revision_internal,
            dummy_variable, software_bootloader_revision, software_build_date,
            dummy_variable, wireless_module_hardware_revision, dummy_variable, dust_software_major, dust_software_minor,
            dust_software_patch, dust_software_build
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];        
        6,  MISC_ERROR, "Device Specific Command error";        
        16, MODE_ERROR, [access_restricted];
    }
}

COMMAND InternalSoftwareRevRead2
{
    NUMBER 169;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status,
            device_serial_num, device_model_num, device_build_date, device_lot_num
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];        
        6,  MISC_ERROR, "Device Specific Command error";        
        16, MODE_ERROR, [access_restricted];
    }
}


VARIABLE io_channel_parity_err_received
{
    LABEL "Parity Errors Received";
    HELP "Count of Parity Errors messages received by the I/O system on this channel";    
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}

VARIABLE io_channel_framing_err_received
{
    LABEL "Framing Errors Received";
    HELP "Count of Framing Errors messages received by the I/O system on this channel";    
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}

VARIABLE io_channel_checkbyte_err_received
{
    LABEL "CheckByte Errors Received";
    HELP "Count of CheckByte Errors messages received by the I/O system on this channel";    
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}

VARIABLE io_channel_gap_err_received
{
    LABEL "Gap Errors Received";
    HELP "Count of Gap Errors messages received by the I/O system on this channel";    
    CLASS DEVICE & DYNAMIC;
    HANDLING READ;
    TYPE UNSIGNED_INTEGER(2);
}


COMMAND read_io_channel_stats_extended   
{
    NUMBER 174;
    OPERATION READ;
    TRANSACTION 0
    {
        REQUEST
        {
            0,
            0
        }
        REPLY
        {
            response_code, device_status,
            0,
            0,
            io_channel_stx_sent,
            io_channel_ack_received,
            io_channel_ostx_received,
            io_channel_oack_received,
            io_channel_back_received,
            io_channel_parity_err_received,
            io_channel_framing_err_received,
            io_channel_checkbyte_err_received,
            io_channel_gap_err_received
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,          [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR, [invalid_selection];
        5,  DATA_ENTRY_ERROR, [too_few_data_bytes_recieved];
        6,  MISC_ERROR,       [xmtr_specific_command_error];
        21, DATA_ENTRY_ERROR, "Invalid I/O card number";
        22, DATA_ENTRY_ERROR, "Invalid channel number";
        32, MODE_ERROR,       [busy];
    }
}


COMMAND SystemUptimeRead
{
    NUMBER 138;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, system_uptime, subdevice_count   
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, "Device Specific Command error";       
        16, MODE_ERROR, [access_restricted];
    }
}


COMMAND MacAddressRead
{
    NUMBER 140;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, device_mac_address
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, "Device Specific Command error";       
        16, MODE_ERROR, [access_restricted];
    }
}


COMMAND ResetToFactory
{
    NUMBER 141;
    OPERATION COMMAND;
    TRANSACTION
    {
        REQUEST
        {
            user_password
        }
        REPLY
        {
            response_code, device_status
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        5,  MISC_ERROR, "reset_specific_command_error";
        7,  MODE_ERROR, [in_write_protect_mode];
        12, MISC_ERROR, "Invalid Password";
        16, MODE_ERROR, [access_restricted];
        32, MODE_ERROR, [busy];
    }
}


COMMAND WriteProtectRead
{
    NUMBER 142;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, write_protect_mode   
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, "Device Specific Command error";       
        16, MODE_ERROR, [access_restricted];
    }
}


COMMAND WriteProtectWrite
{
    NUMBER 143;
    OPERATION WRITE;
    TRANSACTION
    {
        REQUEST
        {
            write_protect_mode
        }
        REPLY
        {
            response_code, device_status, write_protect_mode   
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        2,  DATA_ENTRY_ERROR,   [invalid_selection];
        5,  MISC_ERROR, "Too few data bytes received";
        6,  MISC_ERROR, "Device Specific Command error";       
        16, MODE_ERROR, [access_restricted];
    }
}


COMMAND WirelessInternalStatus
{
    NUMBER 150;
    OPERATION READ;
    TRANSACTION
    {
        REQUEST
        {
        }
        REPLY
        {
            response_code, device_status, 
            dummy_variable, wireless_duty_cycle, 
            dummy_variable, burst_service_time, burst_service_granted, burst_service_requested, 
            dummy_variable, dummy_variable, dummy_variable, dummy_variable, dummy_variable, dummy_variable, 
            dummy_variable, dummy_variable, dummy_variable, dummy_variable, dummy_variable, dummy_variable, 
            last_join_time    
        }
    }
    RESPONSE_CODES
    {
        0,  SUCCESS,    [no_command_specific_errors];
        6,  MISC_ERROR, "Device Specific Command error";       
        16, MODE_ERROR, [access_restricted];
    }
}


METHOD factoryResetMethod
{
    LABEL "Factory reset";
    HELP  "";
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char xmtr_status[MAX_XMTR_STATUS_LEN];
        char disp_string1[60];             
        long x;
        int number_of_tries;
        int slen1;

        slen1 = 60;
        
        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        GET_DEV_VAR_VALUE("Enter Factory Reset Password ", user_password);

        send(141,status);
    
        if (status[STATUS_RESPONSE_CODE])
        {
           display_response_status(141, status[STATUS_RESPONSE_CODE]);   
           process_abort();        /* this will call return_to_normal */
        }

        send(42,status);
        if (status[STATUS_RESPONSE_CODE])
        {
           display_response_status(42, status[STATUS_RESPONSE_CODE]);           
        }
        DELAY_TIME(6);

        RETRY_ON_NO_DEVICE();
        number_of_tries = 0;

        do 
        {
            get_more_status(status, xmtr_status);
        } while (((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY) )                
                && (number_of_tries++ < MAXIMUM_RESET_RETRIES));

        if ((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
        {
            get_dictionary_string(device_stuck_in_device_reset, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
        else if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(48, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);             
            DELAY(2, disp_string1);
        }
        else
        {
            // Need to read out everything that changed
            send(144,status);       // Get Burst Auto Mode
            send(128,status);       // Get Power Mode
            send(74,status);        // Get Master Mode, Retries
            send(7,status);         // Get Polling Addr
            send(142,status);       // Get Write Protect
            send(134,status);       // Get Polling Addr Scan Start, Stop, power-up enab, Time sync            
            send(159,status);       // Get Dust OTAP Setting
            send(146,status);       // Get Loop Switch Parameters

            send(772,status);       // Get Join Mode
            send(774,status);       // Get Network ID            
            send(798,status);       // Get Wireless TX Power

            send(0,status);         // Get Preamble response count
            loop_power_mode = 0;
            adapter_power = 0;
            send(129,status);
            if(status[STATUS_RESPONSE_CODE])
            {
                display_response_status(129, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);             
                DELAY(2, disp_string1);
                process_abort();
            }
            send(147,status);
            if(status[STATUS_RESPONSE_CODE])
            {
                display_response_status(147, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);             
                DELAY(2, disp_string1);
                process_abort();
            }
	   
            ACKNOWLEDGE("System Restored to Factory Defaults");
        }

        process_abort();        /* this will call return_to_normal */
    }
}


METHOD device_master_reset 
{
    LABEL "Perform Master Reset";
    HELP [device_reset_help];
    DEFINITION 
    {
        char status[STATUS_SIZE];
        char xmtr_status[MAX_XMTR_STATUS_LEN];
        char disp_string1[60];             
        long x;
        int number_of_tries;
        int slen1;

        slen1 = 60;

        IGNORE_ALL_RESPONSE_CODES();
        IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_DEVICE_STATUS();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1); 
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);

        get_dictionary_string(about_to_perform_device_reset, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);

        get_dictionary_string(device_reset_in_progress, disp_string1, slen1);
        PUT_MESSAGE(disp_string1);

        send(42, status);

        if(status[STATUS_RESPONSE_CODE])
        {
            display_response_status(42, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);             
            DELAY(2, disp_string1);
            process_abort();
        }
        RETRY_ON_NO_DEVICE();
        number_of_tries = 0;

        get_dictionary_string(device_reset_in_progress, disp_string1, slen1);
        DELAY_TIME(5);

     do 
        {
            get_more_status(status, xmtr_status);
        } while (((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY) )                
                && (number_of_tries++ < MAXIMUM_RESET_RETRIES));

        if ((status[STATUS_RESPONSE_CODE] == DEVICE_BUSY)
                || (status[STATUS_RESPONSE_CODE] == UPDATE_IN_PROGRESS))
        {
            get_dictionary_string(device_stuck_in_device_reset, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }
        else if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(48, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);             
            DELAY(2, disp_string1);
        }
        else if (xmtr_status[0])
        {
            for (x=1; x < 0x100; x <<= 1)
            {
                if (xmtr_status[0] & x)
                {
                    display_xmtr_status(device_specific_status_0, x);
                }
            }
        }
        else
        {
            get_dictionary_string(device_reset_ok, disp_string1, slen1);
            ACKNOWLEDGE(disp_string1);
        }

        process_abort();        /* this will call return_to_normal */
    }
}


IMAGE device_icon
{
    PATH "e0ac_device_icon.gif";
}

IMAGE adapter_pict
{
    PATH "e0ac_adapter.gif";
}

IMAGE alert_status_image
{    
    LABEL "Device Status";
    HELP "Device Status";   

    PATH IF(( extended_fld_device_status & 0x01 ) == 0)
    {
        "e0ac_good.gif";
    }
    ELSE 
    {
        "e0ac_alerts_exist.gif";        
    } 

    LINK alerts_dialog;   
}


IMAGE wireless_status_image
{    
    PATH IF( ( wireless_mode == 0) || ( wireless_mode==6 ) )
    {
       "e0ac_idle.gif";
    }
    ELSE 
    {
        IF(join_status < 0x0010)
        {
            "e0ac_searching.gif";
        }
        ELSE
        { 
            IF(join_status & 0x040)
            {
                "e0ac_join_failed.gif";
            }
            ELSE 
            {
                IF(join_status < 0x400 )
                {
                    "e0ac_joining.gif";
                }
                ELSE
                {
                    "e0ac_joined.gif";
                }
            }
        }
    }
    LINK wireless_status_dlg;
}

MENU device_root_menu
{
    LABEL "Configure";
    STYLE MENU;
    ITEMS
    { 
        overview,
        device_configure_menu,
        diagnostic_root_menu         
    }
}

MENU device_configure_menu
{
    LABEL "Configure";
    STYLE MENU;
    ITEMS
    {
        guided_setup,           // menu 
        manual_setup            // menu 
    }
}



MENU process_variables_root_menu
{
    LABEL "Overview";
    STYLE MENU;
    ITEMS
    {
        overview
    }
}


MENU diagnostic_root_menu
{
    LABEL "Service Tools";
    STYLE MENU;
    ITEMS
    {       
        alerts,        
        status_display,     
        power_time,
        communications,          // menu
        maintenance,             // menu 
        time,                    // menu
        simulate                 // menu        
    }
}


MENU device_info_dialog
{
    LABEL "Device Information";
    STYLE DIALOG;
    ITEMS
    {
        device_info_tab1,
        device_info_tab2,
        device_info_tab3
    }
}

MENU device_info_tab1
{
    LABEL "Device Information";
    STYLE PAGE;
    ITEMS
    {
        device_info_tab1_group1,    
        device_info_tab1_group2,
        device_info_tab2_group5,
        COLUMNBREAK
        adapter_pict(INLINE)       
    }
}

MENU device_info_tab1_group1
{
    LABEL "Identification";
    STYLE GROUP;
    ITEMS
    {
        tag (DISPLAY_VALUE, READ_ONLY),
        longTag(DISPLAY_VALUE, READ_ONLY),        
        final_assembly_number (DISPLAY_VALUE, READ_ONLY)        
    }
}


MENU device_info_tab1_group2
{
    LABEL "Device Identification";
    STYLE GROUP;
    ITEMS
    {
        manufacturer_id (DISPLAY_VALUE, READ_ONLY),
        device_type (DISPLAY_VALUE, READ_ONLY),
        device_mac_address(DISPLAY_VALUE, READ_ONLY),         
        device_serial_num(DISPLAY_VALUE, READ_ONLY),
        device_id (DISPLAY_VALUE, READ_ONLY)
    }
}


MENU device_info_tab2
{
    LABEL "Extended Information";
    STYLE PAGE;
    ITEMS
    {
        device_info_tab2_group1,    
        device_info_tab2_group3,
        COLUMNBREAK,
        device_info_tab2_group2,
        device_info_tab2_group4        
    }
}


MENU device_info_tab2_group1
{
    LABEL "Model Information";
    STYLE GROUP;
    ITEMS
    {
        manufacturer_id (DISPLAY_VALUE, READ_ONLY),
        device_type (DISPLAY_VALUE,READ_ONLY),
        device_model_num(DISPLAY_VALUE,READ_ONLY),
        device_build_date(DISPLAY_VALUE, READ_ONLY),
        device_lot_num(DISPLAY_VALUE, READ_ONLY)   
    }
}


MENU device_info_tab2_group2
{
    LABEL "Information";
    STYLE GROUP;
    ITEMS
    {
        date (DISPLAY_VALUE, READ_ONLY),
        descriptor (DISPLAY_VALUE, READ_ONLY),
        message (DISPLAY_VALUE, READ_ONLY),                   
        country_code (DISPLAY_VALUE,READ_ONLY),
        si_control (DISPLAY_VALUE, READ_ONLY),        
        write_protect_mode (DISPLAY_VALUE, READ_ONLY)
    }
}

MENU device_info_tab2_group3
{
    LABEL "Revision Numbers";
    STYLE GROUP;
    ITEMS
    {
        universal_revision (DISPLAY_VALUE, READ_ONLY),        
        transmitter_revision (DISPLAY_VALUE, READ_ONLY),
        software_revision (DISPLAY_VALUE, READ_ONLY),
        hardware_revision (DISPLAY_VALUE, READ_ONLY)
    }
}


MENU device_info_tab2_group4
{
    LABEL "Software Revision";
    STYLE GROUP;
    ITEMS
    {
        software_revision_internal (DISPLAY_VALUE, READ_ONLY),
        software_bootloader_revision (DISPLAY_VALUE, READ_ONLY),
        software_build_date(DISPLAY_VALUE, READ_ONLY)
    }
}


MENU device_info_tab2_group5
{
    LABEL "DD Version";
    STYLE GROUP;
    ITEMS
    {
        device_description_version2 (DISPLAY_VALUE, READ_ONLY)
    }
}


MENU device_info_tab3
{
    LABEL "Wireless";
    STYLE PAGE;
    ITEMS
    {
        device_info_tab3_group1,
        COLUMNBREAK,
        device_info_tab3_group2        
    }
}

MENU device_info_tab3_group1
{
    LABEL "Wireless Module";
    STYLE GROUP;
    ITEMS
    {
        wireless_module_manufacturer_id1(DISPLAY_VALUE),        
        wireless_module_device_type1(DISPLAY_VALUE),            
        //wireless_module_transmitter_revision(DISPLAY_VALUE),            
        //wireless_module_software_revision(DISPLAY_VALUE),
        wireless_module_hardware_revision(DISPLAY_VALUE) 
    }
}


MENU device_info_tab3_group2
{
    LABEL "Radio Software";
    STYLE GROUP;
    ITEMS
    {
        dust_software_major(DISPLAY_VALUE),
        dust_software_minor(DISPLAY_VALUE),
        dust_software_patch(DISPLAY_VALUE),
        dust_software_build(DISPLAY_VALUE)   
    }
}


MENU overview
{
    LABEL "Overview";
    STYLE WINDOW;
    ITEMS
    {              
        overview_page
    }
}


MENU overview_page
{
    LABEL "Overview";
    STYLE PAGE;
    ITEMS
    {              
        //adapter_pict(INLINE),
        
        overview_pv,
        ROWBREAK,
        //COLUMNBREAK    
        //COLUMNBREAK    
        //overview_status1,
        //overview_status2,
        //COLUMNBREAK
        ROWBREAK        
     
        IF ( subdevice_count == 0x0000 ) 
        {
            sub_device_menu_display_none,
            ROWBREAK
        }        
        IF ( subdevice_count > 0x0000 ) 
        {
            sub_device_menu_display_1,
            ROWBREAK
        }
        IF ( subdevice_count > 0x0001 ) 
        {
            sub_device_menu_display_2,
            ROWBREAK
        }
        IF ( subdevice_count > 0x0002 ) 
        {
            sub_device_menu_display_3,
            ROWBREAK
        }
        IF ( subdevice_count > 0x0003 ) 
        {
            sub_device_menu_display_4,
            ROWBREAK
        }
        IF ( subdevice_count > 0x0004 ) 
        {
            sub_device_menu_display_5,
            ROWBREAK
        }
        IF ( subdevice_count > 0x0005 ) 
        {
            sub_device_menu_display_6,
            ROWBREAK
        }
        IF ( subdevice_count > 0x0006 ) 
        {
            sub_device_menu_display_7,
            ROWBREAK
        }
        IF ( subdevice_count > 0x0007 ) 
        {
            sub_device_menu_display_8,
            ROWBREAK
        }

        IF( subdevice_count == 0x0000 )  
        {
            " ",
            " ",
            " ",            
            ROWBREAK
        }

        IF( subdevice_count == 0x0001 )  
        {
            " ",
            " ",
            " ",            
            ROWBREAK
        }

        IF( subdevice_count == 0x0002 )  
        {
            " ",            
            ROWBREAK
        }
        overview_shortcut        
    }
}


MENU overview_pv
{
    LABEL DEVICE_NAME " Status";
    STYLE GROUP;
    ITEMS
    {   
        adapter_pict(INLINE),
        COLUMNBREAK,
        wireless_mode (DISPLAY_VALUE),        
        PV.DAQ.ANALOG_VALUE (DISPLAY_VALUE), 
        PV.DEVICE_VARIABLE.DIGITAL_VALUE (DISPLAY_VALUE, READ_ONLY),                       
        var1_value(DISPLAY_VALUE, READ_ONLY),        
        var2_value(DISPLAY_VALUE, READ_ONLY),
        "Device Status",
        alert_status_image(INLINE)
        "Wireless Status",
        wireless_status_image(INLINE)
    }
}


MENU overview_status1
{
    LABEL "Device Status";
    STYLE GROUP;
    ITEMS
    {        
        alert_status_image(INLINE)
    }
}

MENU overview_status2
{
    LABEL "Wireless Status";
    STYLE GROUP;
    ITEMS
    {
        wireless_status_image(INLINE)        
        
    }
}


MENU overview_shortcut
{
    LABEL "Shortcuts";
    STYLE GROUP;
    ITEMS
    {
        device_info_dialog,         
        COLUMNBREAK,        
        adapter_config_power,  
        COLUMNBREAK,      
        join_network
    }
}


MENU configure
{
    LABEL "Configure";
    STYLE MENU;
    ITEMS
    {
        guided_setup,           // menu 
        manual_setup            // menu 
    }
}


MENU service_tools
{
    LABEL "Service Tools";
    STYLE MENU;
    ITEMS
    {           
        alerts,        
        status_display,     
        power_time,
        communications,          // menu
        maintenance,             // menu 
        time,                    // menu
        simulate                 // menu
    }
}


MENU communications
{
    LABEL "Communications";
    STYLE WINDOW;   
    ITEMS
    {        
        wireless_status,             
        wireless_statistics,
        message_statistics
    }
}


MENU maintenance
{
    LABEL "Maintenance";
    STYLE WINDOW; 
    ITEMS
    {
        maintenance_page
    }
}

MENU maintenance_page
{
    LABEL "Maintenance";
    STYLE PAGE; 
    ITEMS
    {
        maintenance_group1,
        maintenance_group2,
        maintenance_group3,
        maintenance_group5,
        maintenance_group6,
        maintenance_group7
    }
}


MENU maintenance_group1
{
    LABEL "Master Reset";
    STYLE GROUP; 
    ITEMS
    {
        device_master_reset,                                          // method
        COLUMNBREAK,
        "Resets the " DEVICE_NAME 
    }
}


MENU maintenance_group2
{
    LABEL "Self Test";
    STYLE GROUP; 
    ITEMS
    {
        device_self_test,                                          // method
        COLUMNBREAK,
        "Perform a self test on the " DEVICE_NAME
    }
}


MENU maintenance_group3
{
    LABEL "Factory Reset";
    STYLE GROUP; 
    ITEMS
    {
        factoryResetMethod,                                          // method
        COLUMNBREAK,
        "Resets the "DEVICE_NAME " back to the factory default\nconfiguration"
    }
}


MENU maintenance_group5
{
    LABEL "Advertise to New Devices";
    STYLE GROUP; 
    ITEMS
    {
        active_advertising,                                          // method
        COLUMNBREAK,
        "Advertise to new WirelessHART devices trying\nto join this network"
    }
}


MENU maintenance_group6
{
    LABEL "Calibrate";
    STYLE GROUP; 
    ITEMS
    {
        adapter_loop_cal,                                          // method
        COLUMNBREAK,
        "Calibrate the Loop Current reading of the " DEVICE_NAME
    }
}


MENU maintenance_group7
{
    LABEL "Internal";
    STYLE GROUP; 
    ITEMS
    {
        var3_value(DISPLAY_VALUE)
    }
}


MENU simulate
{
    LABEL "Simulate";
    STYLE WINDOW; 
    ITEMS
    {
        simulate_page 
    }
}

MENU simulate_page
{
    LABEL "Simulate";
    STYLE PAGE; 
    ITEMS
    {
        simulate_group1,     
        simulate_group2,     
        simulate_group3     
    }
}


MENU simulate_group1
{
    LABEL "Simulate Loop Current";
    STYLE GROUP; 
    ITEMS
    {
        device_simulate_loop_current,                                          // method
        COLUMNBREAK,
        "Simulate the loop current reading of the " DEVICE_NAME
    }
}


MENU simulate_group2
{
    LABEL "Simulate Direct Power";
    STYLE GROUP; 
    ITEMS
    {
        device_simulate_direct_power,                                          // method
        COLUMNBREAK,
        "Simulate the direct power reading of the " DEVICE_NAME
    }
}


MENU simulate_group3
{
    LABEL "Simulate Electronics Temperature";
    STYLE GROUP; 
    ITEMS
    {
        device_simulate_temperature,                                          // method
        COLUMNBREAK,
        "Simulate the electronics temperature reading of the " DEVICE_NAME
    }
}


MENU guided_setup
{
    LABEL "Guided Setup";
    STYLE WINDOW;
    ITEMS
    {
        guided_setup_page
    }
}

MENU guided_setup_page
{
    LABEL "Guided Setup";
    STYLE PAGE;
    ITEMS
    {
        guided_setup1,
        guided_setup2
    }
}


MENU guided_setup1
{
    LABEL "Power";
    STYLE GROUP;
    ITEMS
    {
        adapter_config_power,   // method  
        COLUMNBREAK,
        "Set the " DEVICE_NAME " Power Configuration"        
    }
}


MENU guided_setup2
{
    LABEL "Wireless";
    STYLE GROUP;
    ITEMS
    {
        join_network,               // method
        COLUMNBREAK,
        "Set Network ID and Join Key for the WirelessHART network"
    }
}


MENU manual_setup
{
    LABEL "Manual Setup";
    STYLE WINDOW;
    ITEMS
    {
        manual_setup_power,
        manual_setup_wireless,      
        manual_setup_wired, 
        manual_setup_4_20_ma,
        manual_setup_burst1,
        manual_setup_burst2,         
        manual_setup_burst3,      
        manual_setup_event1,
        manual_setup_event2,       
        manual_setup_device        
    }
}


MENU manual_setup_wireless
{
    LABEL "Wireless";
    STYLE PAGE;
    ITEMS
    {
        manual_setup_wireless_group1,
        manual_setup_wireless_group2        
    }
}


MENU manual_setup_wireless_group1
{
    LABEL "Wireless";
    STYLE GROUP;
    ITEMS
    {
        network_id (DISPLAY_VALUE),            
        join_mode (DISPLAY_VALUE), 
        radio_power_output1 (DISPLAY_VALUE),
        COLUMNBREAK,
        "",
        join_network,
        disconnect_network
    }
}

MENU manual_setup_wireless_group2
{
    LABEL "Radio Upgrade";
    STYLE GROUP;
    ITEMS
    {
        dust_otap_setting (DISPLAY_VALUE),            
        COLUMNBREAK,
        "Enable the " DEVICE_NAME " radio module to be upgraded using\nover the air programming"        
    }
}

MENU manual_setup_burst1
{
    LABEL "Burst Settings";
    STYLE PAGE;
    ITEMS
    {
        manual_setup_burst1_group1,  
        burst_menu_0,
        burst_menu_1,
        burst_menu_2,   
        burst_menu_3,   
        burst_menu_4,   
        burst_menu_5, 
        burst_menu_update          
    }
}

MENU manual_setup_burst2
{
    LABEL "Burst Settings 2";
    STYLE PAGE;
    VALIDITY  IF ( license_key_burst > 0x0006 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    { 
        burst_menu_6,
        burst_menu_7,   
        burst_menu_8,   
        burst_menu_9,        
        burst_menu_10,
        burst_menu_11  
    }
}

MENU manual_setup_burst3
{
    LABEL "Burst Settings 3";
    STYLE PAGE;
    VALIDITY  IF ( license_key_burst > 0x000C )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    { 
        burst_menu_12,
        burst_menu_13,
        burst_menu_14,   
        burst_menu_15,   
        burst_menu_16,        
        burst_menu_17        
    }
}


MENU manual_setup_event1
{
    LABEL "Event Notification";
    STYLE PAGE;
    ITEMS
    {         
        event_menu_event0,
        event_menu_event1,
        event_menu_event2,
        event_menu_event3,
        event_menu_event4
    }
}


MENU manual_setup_event2
{
    LABEL "Event Notification 2";
    STYLE PAGE;
    VALIDITY  IF ( license_key_event > 0x0005 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    { 
        event_menu_event5,
        event_menu_event6,
        event_menu_event7,
        event_menu_event8      
    }
}


MENU manual_setup_burst1_group1
{
    LABEL "Auto Burst";
    STYLE GROUP;
    ITEMS
    {
        auto_burst_mode (DISPLAY_VALUE),
        auto_burst_type (DISPLAY_VALUE),
        COLUMNBREAK,
        "\nSet the automatic burst settings"
    }
}


MENU manual_setup_power
{
    LABEL "Power";
    STYLE PAGE;
    ITEMS
    {
        manual_setup_power_group1,
        manual_setup_power_group2,      
        manual_setup_power_group3        
        manual_setup_power_group4        
    }
}

MENU manual_setup_power_group1
{
    LABEL "Guided Power Settings";
    STYLE GROUP;
    ITEMS
    {                     
        adapter_config_power
        COLUMNBREAK,
        "Guided method to configure power settings"
    }
}


MENU manual_setup_power_group2
{
    LABEL DEVICE_NAME " Power Setting";
    STYLE GROUP;
    ITEMS
    {                     
        adapter_power (DISPLAY_VALUE),                
        COLUMNBREAK,        
        "Set the " DEVICE_NAME " Power Mode:\n      Loop Power: Choose the highest voltage step possible\n      Direct Power: Choose the setting based on available power"
    }
}

MENU manual_setup_power_group3
{
    LABEL "Field Device Power Control";
    STYLE GROUP;
    ITEMS
    {   
        loop_power_mode (DISPLAY_VALUE),     
        loop_turn_on_time (DISPLAY_VALUE),                
        loop_idle_time(DISPLAY_VALUE),
        loop_connect_estimated_percent(DISPLAY_VALUE),
        loop_switch_cutoff_en(DISPLAY_VALUE),
        loop_switch_cutoff_voltage(DISPLAY_VALUE),

        "NOTE: These settings are only used when the device\nis wired in the Field Device Power Control Configuration",
        COLUMNBREAK,
        "\nSet the Field Device Power Control Setting\n\nSet the Field Device turn on time\n\nSet the time the Field Device remains powered when\nan external HART master is detected\n\n\n\nEnable the Field Device Cutoff\n\nSet the Field Device Cutoff Voltage"         
    }
}


MENU manual_setup_power_group4
{
    LABEL "External Battery";   
    VALIDITY  IF ( hardware_revision >= 2 )   {TRUE;}   ELSE  {FALSE;}    
    STYLE GROUP;
    ITEMS
    {   
        battery_capacity(DISPLAY_VALUE),
        battery_low_voltage(DISPLAY_VALUE),
        battery_critical_voltage(DISPLAY_VALUE),
        COLUMNBREAK,
        "\nSet the Battery Capacity\n\nSet the Battery Low Voltage\n\nSet the Battery Critical Power Voltage",  
        clear_battery_counters
    }
}


MENU manual_setup_power_status_group1
{
    LABEL "Field Device Power Control Settings";
    STYLE GROUP;
    ITEMS
    {   
        loop_power_mode (DISPLAY_VALUE, READ_ONLY),     
        loop_turn_on_time (DISPLAY_VALUE, READ_ONLY),                
        loop_idle_time(DISPLAY_VALUE,READ_ONLY),
        loop_connect_estimated_percent(DISPLAY_VALUE)
    }
}


MENU manual_setup_power_status_group2
{
    LABEL "Field Device Power Control Stats";
    STYLE GROUP;
    ITEMS
    {   
        loop_connect_on_time(DISPLAY_VALUE),
        loop_connect_off_time(DISPLAY_VALUE),
        loop_connect_on_percent(DISPLAY_VALUE)
             
    }
}


MENU manual_setup_power_status_group3
{
    LABEL "External Battery Settings";  
    VALIDITY  IF ( hardware_revision >= 2 )   {TRUE;}   ELSE  {FALSE;}    
    STYLE GROUP;  
    ITEMS
    {   
        battery_capacity(DISPLAY_VALUE, READ_ONLY),
        battery_low_voltage(DISPLAY_VALUE, READ_ONLY),
        battery_critical_voltage(DISPLAY_VALUE, READ_ONLY)
    }
}


MENU manual_setup_power_status_group4
{
    LABEL "External Battery Stats";  
    VALIDITY  IF ( hardware_revision >= 2 )   {TRUE;}   ELSE  {FALSE;}    
    STYLE GROUP;  
    ITEMS
    {   
        device_power_status (DISPLAY_VALUE, READ_ONLY),     
        battery_life (DISPLAY_VALUE, READ_ONLY),                
        battery_coulomb_curr_rate(DISPLAY_VALUE,READ_ONLY),
        battery_coulomb_total(DISPLAY_VALUE,READ_ONLY),
        battery_coulomb_remain(DISPLAY_VALUE,READ_ONLY),
        battery_change_date(DISPLAY_VALUE,READ_ONLY)
    }
}


MENU manual_setup_wired
{
    LABEL "Wired";
    STYLE PAGE;
    ITEMS
    {
        manual_setup_other_group2,
        manual_setup_wired_group1,
        manual_setup_wired_group2,
        manual_setup_other_group1
    }
}


MENU manual_setup_wired_group1
{
    LABEL "Device Scan Mode";
    STYLE GROUP;
    ITEMS
    {   
        sub_device_powerUp_scan (DISPLAY_VALUE),
        poll_addr_start(DISPLAY_VALUE),
        poll_addr_stop(DISPLAY_VALUE),                
        COLUMNBREAK,
        "Set the " DEVICE_NAME " to scan for wired devices\non power up or reset",        
        "",        
        scan_subdevices
    }
}
       

MENU manual_setup_wired_group2
{
    LABEL "Sub-device Time Sync";
    STYLE GROUP;
    ITEMS
    {   
        sub_device_time_sync_en(DISPLAY_VALUE),        
        COLUMNBREAK,
        "Set to the allow the " DEVICE_NAME " to synchronize wired devices to\nthe WirelessHART time if the device supports a clock"
    }
}


MENU manual_setup_device
{
    LABEL DEVICE_NAME " Information";
    STYLE PAGE;
    ITEMS
    {
        manual_setup_device_group1,
        manual_setup_device_group2,
        COLUMNBREAK,
        device_info_dialog       
    }
}


MENU manual_setup_device_group1
{
    LABEL DEVICE_NAME " Identification";
    STYLE GROUP;
    ITEMS
    {   
        tag(DISPLAY_VALUE),
        longTag(DISPLAY_VALUE),
        descriptor (DISPLAY_VALUE),
        message (DISPLAY_VALUE),
        final_assembly_number (DISPLAY_VALUE),
        date (DISPLAY_VALUE),        
        country_code (DISPLAY_VALUE), 
        si_control (DISPLAY_VALUE),        
        deviceVariables[2].DIGITAL_UNITS (DISPLAY_VALUE)        
    }
}


MENU manual_setup_device_group2
{
    LABEL "Write Protect";
    STYLE GROUP;
    ITEMS
    {   
        write_protect_mode (DISPLAY_VALUE)                    
    }
}


MENU manual_setup_other_group1
{
    LABEL "Polling";
    STYLE GROUP;
    ITEMS
    {   
        polling_address (DISPLAY_VALUE),         
        COLUMNBREAK,
        "Set the HART address for the " DEVICE_NAME "\nwhen using the wired-HART interface"         
    }
}


MENU manual_setup_other_group2
{
    LABEL "HART Master";
    STYLE GROUP;
    ITEMS
    {   
        master_mode (DISPLAY_VALUE),
        retry_count (DISPLAY_VALUE),        
        COLUMNBREAK,        
        "Set the HART master mode for the " DEVICE_NAME "\nwhen communicated with wired devices\n\nSet the number of retries the " DEVICE_NAME " will\nperform for wired communications"       
    }
}


MENU manual_setup_4_20_ma
{
    LABEL "4-20 mA";
    STYLE PAGE;
    ITEMS
    {
        manual_setup_4_20ma_group1   
    }
}


MENU manual_setup_4_20ma_group1
{
    LABEL "4-20 mA";
    STYLE GROUP;
    ITEMS
    {   
        PV.DEVICE_VARIABLE.DIGITAL_UNITS (DISPLAY_VALUE),
        PV.RANGING.TRANSFER_FUNCTION (DISPLAY_VALUE),
        PV.DEVICE_VARIABLE.DAMPING_VALUE (DISPLAY_VALUE),
        PV.RANGING.UPPER_RANGE_VALUE (DISPLAY_VALUE),
        PV.RANGING.LOWER_RANGE_VALUE (DISPLAY_VALUE),
        var0_fault_current_upper (DISPLAY_VALUE),
        var0_prop_current_upper (DISPLAY_VALUE),
        var0_prop_current_lower (DISPLAY_VALUE),
        var0_fault_current_lower (DISPLAY_VALUE),
        pv_lookup_table,
        COLUMNBREAK,  
        "Set the PV Eng Units of the 4-20mA Field Device\n\nSet the 4-20mA Linearization Mode\n\n\nSet the Damping Factor for the PV\n\nOutput assigned to the 20mA Current input\n\n\nOutput assigned to the 4mA Current input\n\nMax current output by the 4-20mA Device when in fault condition\n\n\nUpper limit value of the proportional range\n\nLower limit value of the proportional range\n\n\nMin current output by the 4-20mA Device when in fault condition"    }
}


MENU status_display
{
    LABEL "Device Status";
    STYLE WINDOW;
    ITEMS
    {
        status_display_page
    }
}


MENU status_display_page
{
    LABEL "Device Status";
    STYLE PAGE;
    ITEMS
    {        
        device_status,
        standardized_status_2 (DISPLAY_VALUE),       
        COLUMNBREAK,
        extended_fld_device_status (DISPLAY_VALUE),
        standardized_status_3 (DISPLAY_VALUE),
        COLUMNBREAK,
        standardized_status_0 (DISPLAY_VALUE),
        device_specific_status_0 (DISPLAY_VALUE),       
        device_specific_status_1 (DISPLAY_VALUE), 
        config_change_counter (DISPLAY_VALUE)
    }
}


MENU time
{
    LABEL "Time";
    STYLE WINDOW; 
    ITEMS
    {
        time_page
    }
}

MENU time_page
{
    LABEL "Time";
    STYLE PAGE; 
    ITEMS
    {
        time_group1,
        time_group2
    }
}

MENU time_group1
{
    LABEL "Time";
    STYLE GROUP; 
    ITEMS
    {
        current_date(DISPLAY_VALUE),
        current_time1(DISPLAY_VALUE)
    }
}


MENU time_group2
{
    LABEL "Uptime";
    STYLE GROUP; 
    ITEMS
    {
        system_uptime(DISPLAY_VALUE)
    }
}


MENU power_time
{
    LABEL "Power";
    STYLE WINDOW; 
    ITEMS
    {
        power_time_page
    }
}

MENU power_time_page
{
    LABEL "Power";
    STYLE PAGE; 
    ITEMS
    {
        manual_setup_power_status_group1,
        manual_setup_power_status_group2        
        COLUMNBREAK,
        manual_setup_power_status_group3,
        manual_setup_power_status_group4                        
    }
}


MENU wireless_status_dlg
{
    LABEL "Wireless Status";
    STYLE DIALOG;
    ITEMS
    {
        wireless_status,
        wireless_statistics        
    }
}

MENU wireless_status
{
    LABEL "Wireless Status";
    STYLE PAGE;
    ITEMS
    {
        wireless_status_group1,
        wireless_status_group2,
        COLUMNBREAK,
        wireless_status_group3        
    }
}

MENU wireless_status_group1
{
    LABEL "Join Details";
    STYLE GROUP;
    ITEMS
    {
        wireless_mode (DISPLAY_VALUE),
        join_mode (DISPLAY_VALUE, READ_ONLY),          
        wireless_duty_cycle(DISPLAY_VALUE),
        last_join_time(DISPLAY_VALUE),
        advertisements (DISPLAY_VALUE),
        neighbor_count (DISPLAY_VALUE),
        join_attempts (DISPLAY_VALUE)
    }
}


MENU wireless_status_group2
{
    LABEL "Burst Service Status";
    STYLE GROUP;
    ITEMS
    {
        burst_service_requested (DISPLAY_VALUE),
        burst_service_granted (DISPLAY_VALUE)     
    }
}

MENU wireless_status_group3
{
    LABEL "Join Status";
    STYLE GROUP;
    ITEMS
    {
        join_status[0x0001] (DISPLAY_VALUE),        
        join_status[0x0004] (DISPLAY_VALUE),
        join_status[0x0008] (DISPLAY_VALUE),
        join_status[0x0010] (DISPLAY_VALUE),
        join_status[0x0020] (DISPLAY_VALUE),
        join_status[0x0040] (DISPLAY_VALUE),
        join_status[0x0080] (DISPLAY_VALUE),
        join_status[0x0100] (DISPLAY_VALUE),
        join_status[0x0200] (DISPLAY_VALUE),
        join_status[0x0400] (DISPLAY_VALUE)
    }
}


MENU wireless_statistics
{
    LABEL "Wireless Statistics";
    STYLE PAGE;
    ITEMS
    {
        wireless_statistics_group
    }
}


MENU wireless_statistics_group
{
    LABEL "Wireless Statistics";
    STYLE GROUP;
    ITEMS
    {
        generated_packets (DISPLAY_VALUE),
        terminated_packets (DISPLAY_VALUE),
        detected_crc_errors (DISPLAY_VALUE),
        detected_dll_mic_failures (DISPLAY_VALUE),
        detected_nl_mic_failures (DISPLAY_VALUE),
        not_received_nonce_counter_values (DISPLAY_VALUE)
    }
}


MENU message_statistics
{
    LABEL "Wired Statistics";
    STYLE PAGE;
    ITEMS
    {
       AdapterStats,                 // menu
       COLUMNBREAK,
       message_statistics_master           
    }
}


MENU AdapterStats
{
    LABEL "Wired Slave Statistics";
    STYLE GROUP;
    ITEMS
    {
       STX_count(DISPLAY_VALUE),
       ACK_count(DISPLAY_VALUE),
       BACK_count(DISPLAY_VALUE)
    }
}


MENU message_statistics_master
{
    LABEL "Wired Master Statistics";
    STYLE GROUP;
    ITEMS
    {
       io_channel_stx_sent(DISPLAY_VALUE),
       io_channel_ack_received(DISPLAY_VALUE),
       io_channel_ostx_received(DISPLAY_VALUE),
       io_channel_oack_received(DISPLAY_VALUE),
       io_channel_back_received(DISPLAY_VALUE),       
       io_channel_parity_err_received(DISPLAY_VALUE),
       io_channel_framing_err_received(DISPLAY_VALUE),
       io_channel_checkbyte_err_received(DISPLAY_VALUE),
       io_channel_gap_err_received(DISPLAY_VALUE)       
    }
}


MENU root_menu
{
    LABEL "Online";
    ITEMS
    {
        overview_tree,               // menu 
        configure_tree,              // menu 
        service_tools_tree           // menu 
    }
}

MENU overview_tree
{
    LABEL "Overview";    
    ITEMS
    {   
        alerts,
        wireless_mode (DISPLAY_VALUE),
        PV.DAQ.ANALOG_VALUE (DISPLAY_VALUE),
        PV.DEVICE_VARIABLE.DIGITAL_VALUE (DISPLAY_VALUE, READ_ONLY),        
        var1_value(DISPLAY_VALUE, READ_ONLY),        
        var2_value(DISPLAY_VALUE, READ_ONLY),          
        subdevice_info_tree1,
        subdevice_info_tree2,
        subdevice_info_tree3,
        subdevice_info_tree4,
        subdevice_info_tree5,
        subdevice_info_tree6,
        subdevice_info_tree7,
        subdevice_info_tree8                      
    }
}


MENU device_info_main_tree
{
    LABEL "Device Information";
    ITEMS
    {
        tag (DISPLAY_VALUE),
        longTag(DISPLAY_VALUE),   
        device_info_tree,
        wireless_info_tree
    }
}


MENU device_info_tree
{
    LABEL "Device";
    ITEMS
    {
        manufacturer_id (DISPLAY_VALUE, READ_ONLY),
        device_type (DISPLAY_VALUE, READ_ONLY),
        device_mac_address(DISPLAY_VALUE, READ_ONLY),         
        device_serial_num(DISPLAY_VALUE, READ_ONLY),
        device_model_num(DISPLAY_VALUE,READ_ONLY),
        device_build_date(DISPLAY_VALUE, READ_ONLY),
        device_lot_num(DISPLAY_VALUE, READ_ONLY)   
        device_id (DISPLAY_VALUE, READ_ONLY),
        universal_revision (DISPLAY_VALUE, READ_ONLY),        
        transmitter_revision (DISPLAY_VALUE, READ_ONLY),
        hardware_revision (DISPLAY_VALUE, READ_ONLY),        
        software_revision (DISPLAY_VALUE, READ_ONLY),      
        software_revision_internal (DISPLAY_VALUE, READ_ONLY),
        software_bootloader_revision (DISPLAY_VALUE, READ_ONLY),
        software_build_date(DISPLAY_VALUE, READ_ONLY)
        device_description_version2 (DISPLAY_VALUE, READ_ONLY)
        final_assembly_number (DISPLAY_VALUE),
        date (DISPLAY_VALUE),
        descriptor (DISPLAY_VALUE),
        message (DISPLAY_VALUE),                   
        country_code (DISPLAY_VALUE),
        si_control (DISPLAY_VALUE),        
        deviceVariables[2].DIGITAL_UNITS (DISPLAY_VALUE),            
        write_protect_mode (DISPLAY_VALUE)
    }
}


EDIT_DISPLAY pv_lookup_table
{
    LABEL "PV Linearization Table";
    DISPLAY_ITEMS
    {
        pv_lookup_ma_value1,
        pv_lookup_eng_value1, 
        pv_lookup_ma_value2,
        pv_lookup_eng_value2,
        pv_lookup_ma_value3,
        pv_lookup_eng_value3,
        pv_lookup_ma_value4,
        pv_lookup_eng_value4,
        pv_lookup_ma_value5,
        pv_lookup_eng_value5,
        pv_lookup_ma_value6,
        pv_lookup_eng_value6,
        pv_lookup_ma_value7,
        pv_lookup_eng_value7,
        pv_lookup_ma_value8,
        pv_lookup_eng_value8,
        pv_lookup_ma_value9,
        pv_lookup_eng_value9,
        pv_lookup_ma_value10,
        pv_lookup_eng_value10,
        pv_lookup_ma_value11,
        pv_lookup_eng_value11,
        pv_lookup_ma_value12,
        pv_lookup_eng_value12,
        pv_lookup_ma_value13,
        pv_lookup_eng_value13,
        pv_lookup_ma_value14,
        pv_lookup_eng_value14,
        pv_lookup_ma_value15,
        pv_lookup_eng_value15,
        pv_lookup_ma_value16,
        pv_lookup_eng_value16,
        pv_lookup_ma_value17,
        pv_lookup_eng_value17,
        pv_lookup_ma_value18,
        pv_lookup_eng_value18,
        pv_lookup_ma_value19,
        pv_lookup_eng_value19,
        pv_lookup_ma_value20,
        pv_lookup_eng_value20,
        pv_lookup_ma_value21,
        pv_lookup_eng_value21,
        pv_lookup_ma_value22,
        pv_lookup_eng_value22,
        pv_lookup_ma_value23,
        pv_lookup_eng_value23,
        pv_lookup_ma_value24,
        pv_lookup_eng_value24,
        pv_lookup_ma_value25,
        pv_lookup_eng_value25,
        pv_lookup_ma_value26,
        pv_lookup_eng_value26,
        pv_lookup_ma_value27,
        pv_lookup_eng_value27,
        pv_lookup_ma_value28,
        pv_lookup_eng_value28,
        pv_lookup_ma_value29,
        pv_lookup_eng_value29,
        pv_lookup_ma_value30,
        pv_lookup_eng_value30,
        pv_lookup_ma_value31,
        pv_lookup_eng_value31,
        pv_lookup_ma_value32,
        pv_lookup_eng_value32
    }
    EDIT_ITEMS 
    { 
        pv_lookup1,
        pv_lookup2,
        pv_lookup3,
        pv_lookup4,
        pv_lookup5,
        pv_lookup6,
        pv_lookup7,
        pv_lookup8,
        pv_lookup9,
        pv_lookup10,
        pv_lookup11,
        pv_lookup12,
        pv_lookup13,
        pv_lookup14,
        pv_lookup15,
        pv_lookup16,
        pv_lookup17,
        pv_lookup18,
        pv_lookup19,
        pv_lookup20,
        pv_lookup21,
        pv_lookup22,
        pv_lookup23,
        pv_lookup24,
        pv_lookup25,
        pv_lookup26,
        pv_lookup27,
        pv_lookup28,
        pv_lookup29,
        pv_lookup30,
        pv_lookup31,
        pv_lookup32
    }
}


MENU wireless_info_tree
{
    LABEL "Wireless";
    ITEMS
    {
        wireless_module_manufacturer_id1(DISPLAY_VALUE),        
        wireless_module_device_type1(DISPLAY_VALUE),            
        //wireless_module_transmitter_revision(DISPLAY_VALUE),            
        //wireless_module_software_revision(DISPLAY_VALUE),
        wireless_module_hardware_revision(DISPLAY_VALUE) 
        dust_software_major(DISPLAY_VALUE),
        dust_software_minor(DISPLAY_VALUE),
        dust_software_patch(DISPLAY_VALUE),
        dust_software_build(DISPLAY_VALUE)  
    }
}


MENU configure_tree
{
    LABEL "Configure";    
    ITEMS
    {
        guided_setup_tree,           // menu 
        manual_setup_tree            // menu 
    }
}


MENU service_tools_tree
{
    LABEL "Service Tools";
    ITEMS
    {           
        alerts_tree,        
        status_display_tree,     
        power_time_tree,
        communications_tree,
        maintenance_tree,
        time_tree,            
        simulate_tree           
    }
}


MENU status_display_tree
{
    LABEL "Device Status";    
    ITEMS
    {
        device_status (DISPLAY_VALUE),
        standardized_status_2 (DISPLAY_VALUE),               
        extended_fld_device_status (DISPLAY_VALUE),
        standardized_status_3 (DISPLAY_VALUE),        
        standardized_status_0 (DISPLAY_VALUE),
        device_specific_status_0 (DISPLAY_VALUE),       
        device_specific_status_1 (DISPLAY_VALUE),       
        config_change_counter (DISPLAY_VALUE)
    }
}


MENU power_time_tree
{
    LABEL "Power";    
    ITEMS
    {   
        loop_power_mode (DISPLAY_VALUE, READ_ONLY),     
        loop_turn_on_time (DISPLAY_VALUE, READ_ONLY),                
        loop_idle_time(DISPLAY_VALUE,READ_ONLY),
        loop_connect_estimated_percent(DISPLAY_VALUE),
        loop_connect_on_time(DISPLAY_VALUE),
        loop_connect_off_time(DISPLAY_VALUE),
        loop_connect_on_percent(DISPLAY_VALUE)        
        power_battery_tree        
    }
}

MENU power_battery_tree
{
    LABEL "External Battery";    
    VALIDITY  IF ( hardware_revision >= 2 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    {   
        battery_capacity(DISPLAY_VALUE, READ_ONLY),
        battery_low_voltage(DISPLAY_VALUE, READ_ONLY),
        battery_critical_voltage(DISPLAY_VALUE, READ_ONLY),
        device_power_status (DISPLAY_VALUE, READ_ONLY),     
        battery_life (DISPLAY_VALUE, READ_ONLY),                
        battery_coulomb_curr_rate(DISPLAY_VALUE,READ_ONLY),
        battery_coulomb_total(DISPLAY_VALUE,READ_ONLY),
        battery_coulomb_remain(DISPLAY_VALUE,READ_ONLY),
        battery_change_date(DISPLAY_VALUE,READ_ONLY)
    }
}


MENU simulate_tree
{
    LABEL "Simluate";
    ITEMS
    {
        device_simulate_loop_current
        device_simulate_direct_power, 
        device_simulate_temperature
    }
}



MENU time_tree
{
    LABEL "Time";
    ITEMS
    {
        current_date(DISPLAY_VALUE),
        current_time1(DISPLAY_VALUE),
        system_uptime(DISPLAY_VALUE)
    }
}


MENU maintenance_tree
{
    LABEL "Maintenance";
    ITEMS
    {
        device_master_reset,
        device_self_test,
        factoryResetMethod,
        active_advertising,
        adapter_loop_cal,
        var3_value(DISPLAY_VALUE)
    }
}


MENU communications_tree
{
    LABEL "Communications";
    ITEMS
    {
        wireless_status_tree,             
        wireless_statistics_tree,
        message_statistics_tree,
        message_statistics_tree1
    }
}


MENU wireless_status_tree
{
    LABEL "Wireless Status";    
    ITEMS
    {
        wireless_mode (DISPLAY_VALUE),
        join_status(DISPLAY_VALUE),   
        join_mode (DISPLAY_VALUE, READ_ONLY),          
        wireless_duty_cycle(DISPLAY_VALUE),
        last_join_time(DISPLAY_VALUE),
        advertisements (DISPLAY_VALUE),
        neighbor_count (DISPLAY_VALUE),
        join_attempts (DISPLAY_VALUE),
        burst_service_requested (DISPLAY_VALUE),
        burst_service_granted (DISPLAY_VALUE)   

    }
}


MENU wireless_statistics_tree
{
    LABEL "Wireless Statistics";    
    ITEMS
    {
        generated_packets (DISPLAY_VALUE),
        terminated_packets (DISPLAY_VALUE),
        detected_crc_errors (DISPLAY_VALUE),
        detected_dll_mic_failures (DISPLAY_VALUE),
        detected_nl_mic_failures (DISPLAY_VALUE),
        not_received_nonce_counter_values (DISPLAY_VALUE)
    }
}


MENU message_statistics_tree
{
    LABEL "Wired Slave Statistics";
    ITEMS
    {
       STX_count(DISPLAY_VALUE),
       ACK_count(DISPLAY_VALUE),
       BACK_count(DISPLAY_VALUE)
    }
}


MENU message_statistics_tree1
{
    LABEL "Wired Master Statistics";
    ITEMS
    {
       io_channel_stx_sent(DISPLAY_VALUE),
       io_channel_ack_received(DISPLAY_VALUE),
       io_channel_ostx_received(DISPLAY_VALUE),
       io_channel_oack_received(DISPLAY_VALUE),
       io_channel_back_received(DISPLAY_VALUE)       
       io_channel_parity_err_received(DISPLAY_VALUE),
       io_channel_framing_err_received(DISPLAY_VALUE),
       io_channel_checkbyte_err_received(DISPLAY_VALUE),
       io_channel_gap_err_received(DISPLAY_VALUE)       
    }
}


MENU guided_setup_tree
{
    LABEL "Guided Setup";
    ITEMS
    {
        adapter_config_power,   // method  
        join_network
    }
}


MENU manual_setup_tree
{
    LABEL "Manual Setup";
    ITEMS
    {
        manual_setup_power_tree,
        manual_setup_wireless_tree,           
        manual_setup_wired_tree        
        manual_setup_4_20_ma_tree,
        manual_setup_burst_tree,
        manual_setup_event_tree,           
        device_info_main_tree
    }
}


MENU manual_setup_power_tree
{
    LABEL "Power";
    ITEMS
    {
        adapter_config_power,
        adapter_power (DISPLAY_VALUE), 
        loop_power_mode (DISPLAY_VALUE), 
        manual_setup_power_tree1        
        manual_setup_power_tree2        
    }
}


MENU manual_setup_power_tree1
{
    LABEL "Power Control Settings";
    VALIDITY  IF ( ( license_key_power & 0x0300 ) == 0x0300 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    {           
        loop_turn_on_time (DISPLAY_VALUE),                
        loop_idle_time(DISPLAY_VALUE),
        loop_connect_estimated_percent(DISPLAY_VALUE),
        loop_switch_cutoff_en(DISPLAY_VALUE),
        loop_switch_cutoff_voltage(DISPLAY_VALUE)

    }
}

MENU manual_setup_power_tree2
{
    LABEL "External Battery Settings";    
    VALIDITY  IF ( hardware_revision >= 2 )   {TRUE;}   ELSE  {FALSE;}    
    ITEMS
    {           
        battery_capacity(DISPLAY_VALUE),
        battery_low_voltage(DISPLAY_VALUE),
        battery_critical_voltage(DISPLAY_VALUE),
        clear_battery_counters
    }
}


MENU manual_setup_wireless_tree
{
    LABEL "Wireless";    
    ITEMS
    {
        join_network,
        disconnect_network,
        network_id (DISPLAY_VALUE),            
        join_mode (DISPLAY_VALUE), 
        radio_power_output1 (DISPLAY_VALUE), 
        dust_otap_setting (DISPLAY_VALUE)
    }
}


MENU manual_setup_wired_tree
{
    LABEL "Wired";    
    ITEMS
    {
        master_mode (DISPLAY_VALUE),
        retry_count (DISPLAY_VALUE),  
        polling_address (DISPLAY_VALUE), 
        sub_device_time_sync_en(DISPLAY_VALUE), 
        sub_device_powerUp_scan (DISPLAY_VALUE),
        poll_addr_start(DISPLAY_VALUE),
        poll_addr_stop(DISPLAY_VALUE),  
        scan_subdevices
    }
}


MENU manual_setup_4_20_ma_tree
{
    LABEL "4-20 mA";
    ITEMS
    {
        PV.DEVICE_VARIABLE.DIGITAL_UNITS (DISPLAY_VALUE),
        PV.RANGING.TRANSFER_FUNCTION (DISPLAY_VALUE),
        PV.DEVICE_VARIABLE.DAMPING_VALUE (DISPLAY_VALUE),
        PV.RANGING.UPPER_RANGE_VALUE (DISPLAY_VALUE),
        PV.RANGING.LOWER_RANGE_VALUE (DISPLAY_VALUE),
        var0_fault_current_upper (DISPLAY_VALUE),
        var0_prop_current_upper (DISPLAY_VALUE),
        var0_prop_current_lower (DISPLAY_VALUE),
        var0_fault_current_lower (DISPLAY_VALUE),
        pv_lookup_table
    }
}


MENU manual_setup_burst_tree
{
    LABEL "Burst Settings";
    ITEMS
    {
        auto_burst_mode (DISPLAY_VALUE), 
        auto_burst_type (DISPLAY_VALUE),
        burst_menu_tree_0,
        burst_menu_tree_1,
        burst_menu_tree_2,
        burst_menu_tree_3,
        burst_menu_tree_4,
        burst_menu_tree_5,        
        burst_menu_tree_6,
        burst_menu_tree_7,
        burst_menu_tree_8,
        burst_menu_tree_9,
        burst_menu_tree_10,
        burst_menu_tree_11,
        burst_menu_tree_12,
        burst_menu_tree_13,
        burst_menu_tree_14,
        burst_menu_tree_15,
        burst_menu_tree_16,
        burst_menu_tree_17  
    }
}

MENU manual_setup_event_tree
{
    LABEL "Event Settings";
    ITEMS
    {             
        event_menu_tree_0,       
        event_menu_tree_1
        event_menu_tree_2,
        event_menu_tree_3,
        event_menu_tree_4,       
        event_menu_tree_5,
        event_menu_tree_6,
        event_menu_tree_7,
        event_menu_tree_8   
    }
}


METHOD adapter_loop_cal
{
    LABEL "Loop Current Calibration";
    HELP "Loop Current Calibration";
/*  APPHELP "This type of calibration is usually used to compensate for"
            " drifting or aging output electronics"; */
    DEFINITION
    {
        char status[STATUS_SIZE];
        char disp_string1[60];
        char disp_string2[60];
        long var_id[1];
        int retry;
        int slen1;
        int slen2;
        float fOffset;

        slen1 = 60;
        slen2 = 60;

        IGNORE_ALL_DEVICE_STATUS();
        IGNORE_ALL_RESPONSE_CODES();
        XMTR_IGNORE_ALL_DEVICE_STATUS();
        XMTR_IGNORE_ALL_RESPONSE_CODES();

        get_dictionary_string(remove_loop_from_auto, disp_string1, slen1);
        ACKNOWLEDGE(disp_string1);
        add_abort_method(return_to_normal);
        
        ACKNOWLEDGE("Connect current source to Adapter");   
        ACKNOWLEDGE("Set current source to 4.0mA");

        assign_float(PV.DAQ.ANALOG_VALUE,4.0);
        send(40, status);

        if (status[STATUS_RESPONSE_CODE])
        {
            display_response_status(40,status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        remove_abort_method(return_to_normal);
        add_abort_method(leave_fixed_current_mode);

        var_id[0] = VARID(PV.DAQ.ANALOG_VALUE);
        fOffset = var1_value;
        fOffset *= 0.002;        

        do
        {            
            GET_DEV_VAR_VALUE("Enter exact current source value", PV.DAQ.ANALOG_VALUE);
            
            PV.DAQ.ANALOG_VALUE += fOffset;           

            send(45, status);

            assign_float(PV.DAQ.ANALOG_VALUE, 4.0);  /*Put the previous value back */

            if (( status[STATUS_RESPONSE_CODE] == 3) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if (status[STATUS_RESPONSE_CODE])
            {
                display_response_status(45, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_otpt_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);
        
        ACKNOWLEDGE("Set current source to 20.0mA");

        assign_float(PV.DAQ.ANALOG_VALUE, 20.0);

        send(40, status);

        if ( status[STATUS_RESPONSE_CODE])
        {
            display_response_status(40, status[STATUS_RESPONSE_CODE]);
            get_dictionary_string(abort_last_error, disp_string1, slen1);
            DELAY(2, disp_string1);
            process_abort();
        }

        do
        {            
            GET_DEV_VAR_VALUE("Enter exact current source value", PV.DAQ.ANALOG_VALUE);

            PV.DAQ.ANALOG_VALUE += fOffset;           

            send(46,status);

            assign_float(PV.DAQ.ANALOG_VALUE, 20.0);  /*Put the previous value back */

            if ((status[STATUS_RESPONSE_CODE] == 3 ) || (status[STATUS_RESPONSE_CODE] == 4))
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                retry = 1;
            }
            else if ( status[STATUS_RESPONSE_CODE])
            {
                display_response_status(46, status[STATUS_RESPONSE_CODE]);
                get_dictionary_string(abort_last_error, disp_string1, slen1);
                DELAY(2, disp_string1);
                process_abort();
            }
            else
            {
                get_dictionary_string(fld_dev_otpt_equal_ref_meter, disp_string1, slen1);
                get_dictionary_string(yes_no, disp_string2, slen2);
                retry = select_from_list(disp_string1, var_id, disp_string2);
            }
        } while (retry);

        process_abort() ;
    }
}
